/**
Bryn Mawr College, Department of Computer Science <br>
Conway's Game of Life <br>
Use mouse press/drag to toggle/set cell state <br>
Use space bar to start/stop simulation <br>
Must click in screen first before keys will function.
*/


// Flags and game parameters
int N = 50;               // Grid Dimension
int cellSize = 10;        // Size of cell
int curr = 0;             // Current grid index
int next = 1;             // Next grid index
boolean running = false;  // True if running

// Two cell grids for current and next step
boolean[][][] cell = new boolean[N][N][2];

void setup() {
  size(500, 500);
  rectMode(CORNER);
  randomizeGrid(curr);
  frameRate(30);
}

void draw() {
  // Draw the background
  drawBackground();

  // Draw current live cells
  drawLiveCells();

  // If game is running, advance it
  if (running) advanceGame();
}

void keyPressed() {
  if (key == ' ') {
    
    // Toggle running state
    running = !running;
    
    // Reduce frame rate when running.
    // Increase frame rate when idle.
    if (running) {
      frameRate(5);
    } else {
      frameRate(30);
    }
  }
}

void mousePressed() {
  // When mouse is pressed
  // toggle state of grid
  int i = int(mouseX/cellSize);
  int j = int(mouseY/cellSize);
  cell[i][j][curr] = !cell[i][j][curr];
}

void mouseDragged() {
  // When mouse is dragged
  // create lives in cells
  int i = int(mouseX/cellSize);
  int j = int(mouseY/cellSize);
  cell[i][j][curr] = true;
}

// Draw the game background
void drawBackground() {
  // Clear background
  background(255);
  stroke(240);

  // Draw grid lines
  for (int x=0; x<=width; x=x+cellSize) {
    line(x, 0, x, height);
  }
  for (int y=0; y<=width; y=y+cellSize) {
    line(0, y, width, y);
  }
}

// Draw the current live cells
void drawLiveCells() {
  for (int i=0; i<N; i++) {
    for (int j=0; j<N; j++) {
      noStroke();
      if (cell[i][j][curr] == true) {
        fill(0);
        int x = i*cellSize;
        int y = j*cellSize;
        rect(x, y, cellSize, cellSize);
      }
    }
  }
}

// Randomly set cells alive
void randomizeGrid(int c) {
  for (int i=0; i<N; i++) {
    for (int j=0; j<N; j++) {
      float r = random(0, 100);
      if (r < 20) cell[i][j][c] = true;
    }
  }
}

void advanceGame() {
  //http://en.wikipedia.org/wiki/Conway%27s_game_of_life
  //1. Any live cell with fewer than two live neighbours dies, as if caused by under-population.
  //2. Any live cell with two or three live neighbours lives on to the next generation.
  //3. Any live cell with more than three live neighbours dies, as if by overcrowding.
  //4. Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.

  // Loop over all locations
  for (int i=0; i<N; i++) {
    for (int j=0; j<N; j++) {

      // Count location neighbors
      int neighbors = countNeighbors(i, j);

      // If the cell is alive ...
      if (cell[i][j][curr] == true) {

        if ( neighbors < 2 ) {
          cell[i][j][next] = false;
        } 
        else if (neighbors >=2 && neighbors <=3) {
          cell[i][j][next] = true;
        } 
        else if (neighbors >3) {
          cell[i][j][next] = false;
        }

        // If cell is dead ...
      } 
      else {

        if (neighbors == 3) {
          cell[i][j][next] = true;
        } 
        else {
          cell[i][j][next] = false;
        }
      }
    }
  }

  // Swap grid indexes
  int tmp = curr;
  curr = next;
  next = tmp;
}

// Count the number of neighbors of current cell
int countNeighbors(int i, int j) {
  int count = 0;
  for (int ii=i-1; ii<=i+1; ii++) {
    for (int jj=j-1; jj<=j+1; jj++) {

      // Wrap edges of board
      int iii = (ii + N) % N;
      int jjj = (jj + N) % N;

      // Skip center location, which is not a neighbor
      if (iii!=i || jjj!=j)
        if ( cell[iii][jjj][curr] == true )
          count++;
    }
  }
  return count;
}


